@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 
 import strawberry
 from strawberry import UNSET
@@ -9,6 +9,6 @@ from strawberry.scalars import JSON
 @strawberry.input
 class AddSpansToDatasetInput:
     dataset_id: GlobalID
-    span_ids: List[GlobalID]
+    span_ids: list[GlobalID]
     dataset_version_description: Optional[str] = UNSET
     dataset_version_metadata: Optional[JSON] = UNSET
